@@ -20,13 +20,15 @@
 #include "kservice.h"
 #include "kservice_p.h"
 #include "kmimetypefactory_p.h"
+#include "ksycoca.h"
+#include "ksycoca_p.h"
 
 #include <qplatformdefs.h>
 
-#include <QtCore/QCharRef>
-#include <QtCore/QFile>
-#include <QtCore/QDir>
-#include <QtCore/QMap>
+#include <QCharRef>
+#include <QFile>
+#include <QDir>
+#include <QMap>
 #include <QCoreApplication>
 #include <qmimedatabase.h>
 
@@ -38,8 +40,10 @@
 #include <qstandardpaths.h>
 #include <qdebug.h>
 
-#include "kservicefactory.h"
-#include "kservicetypefactory.h"
+#include "kservicefactory_p.h"
+#include "kservicetypefactory_p.h"
+#include "kserviceutil_p.h"
+#include "servicesdebug.h"
 
 QDataStream &operator<<(QDataStream &s, const KService::ServiceTypeAndPreference &st)
 {
@@ -61,18 +65,18 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     const KConfigGroup desktopGroup = const_cast<KDesktopFile *>(config)->desktopGroup();
     QMap<QString, QString> entryMap = desktopGroup.entryMap();
 
-    entryMap.remove(QLatin1String("Encoding")); // reserved as part of Desktop Entry Standard
-    entryMap.remove(QLatin1String("Version"));  // reserved as part of Desktop Entry Standard
+    entryMap.remove(QStringLiteral("Encoding")); // reserved as part of Desktop Entry Standard
+    entryMap.remove(QStringLiteral("Version"));  // reserved as part of Desktop Entry Standard
 
     q->setDeleted(desktopGroup.readEntry("Hidden", false));
-    entryMap.remove(QLatin1String("Hidden"));
+    entryMap.remove(QStringLiteral("Hidden"));
     if (q->isDeleted()) {
         m_bValid = false;
         return;
     }
 
     m_strName = config->readName();
-    entryMap.remove(QLatin1String("Name"));
+    entryMap.remove(QStringLiteral("Name"));
     if (m_strName.isEmpty()) {
         // Try to make up a name.
         m_strName = entryPath;
@@ -80,21 +84,21 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         m_strName = m_strName.mid(i + 1);
         i = m_strName.lastIndexOf(QLatin1Char('.'));
         if (i != -1) {
-            m_strName = m_strName.left(i);
+            m_strName.truncate(i);
         }
     }
 
     m_strType = config->readType();
-    entryMap.remove(QLatin1String("Type"));
+    entryMap.remove(QStringLiteral("Type"));
     if (m_strType.isEmpty()) {
         /*kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
           << " has no Type=... entry."
           << " It should be \"Application\" or \"Service\"";
           m_bValid = false;
           return;*/
-        m_strType = QString::fromLatin1("Application");
+        m_strType = QStringLiteral("Application");
     } else if (m_strType != QLatin1String("Application") && m_strType != QLatin1String("Service")) {
-        qWarning() << "The desktop entry file " << entryPath
+        qCWarning(SERVICES) << "The desktop entry file " << entryPath
                    << " has Type=" << m_strType
                    << " instead of \"Application\" or \"Service\"";
         m_bValid = false;
@@ -106,12 +110,12 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     // behaviour still happens if the "e" flag is set, maintaining
     // backwards compatibility.
     m_strExec = desktopGroup.readEntry("Exec", QString());
-    entryMap.remove(QLatin1String("Exec"));
+    entryMap.remove(QStringLiteral("Exec"));
 
     if (m_strType == QLatin1String("Application")) {
         // It's an application? Should have an Exec line then, otherwise we can't run it
         if (m_strExec.isEmpty()) {
-            qWarning() << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but no Exec line";
+            qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but no Exec line";
             m_bValid = false;
             return;
         }
@@ -129,9 +133,9 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     if ((m_strType == QLatin1String("Application")) &&
             (resource != QStandardPaths::ApplicationsLocation) &&
             !absPath) {
-        qWarning() << "The desktop entry file" << entryPath
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath
                    << "has Type=" << m_strType << "but is located under \"" << QStandardPaths::displayName(resource)
-                   << "\" instead of \"apps\"";
+                   << "\" instead of \"Applications\"";
         m_bValid = false;
         return;
     }
@@ -139,73 +143,70 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     if ((m_strType == QLatin1String("Service")) &&
             (resource != QStandardPaths::GenericDataLocation) &&
             !absPath) {
-        qWarning() << "The desktop entry file" << entryPath
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath
                    << "has Type=" << m_strType << "but is located under \"" << QStandardPaths::displayName(resource)
-                   << "\" instead of \"shared data\"";
+                   << "\" instead of \"Shared Data\"/kservices5";
         m_bValid = false;
         return;
     }
 
-    QString _name = entryPath;
-    int pos = _name.lastIndexOf(QLatin1Char('/'));
-    if (pos != -1) {
-        _name = _name.mid(pos + 1);
-    }
-    pos = _name.indexOf(QLatin1Char('.'));
-    if (pos != -1) {
-        _name = _name.left(pos);
-    }
+    // entryPath To desktopEntryName
+    // (e.g. "/home/x/.qttest/share/kservices5/fakepart2.desktop" --> "fakepart2")
+    QString _name = KServiceUtilPrivate::completeBaseName(entryPath);
 
     m_strIcon = config->readIcon();
-    entryMap.remove(QLatin1String("Icon"));
+    entryMap.remove(QStringLiteral("Icon"));
     m_bTerminal = desktopGroup.readEntry("Terminal", false);  // should be a property IMHO
-    entryMap.remove(QLatin1String("Terminal"));
+    entryMap.remove(QStringLiteral("Terminal"));
     m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions");   // should be a property IMHO
-    entryMap.remove(QLatin1String("TerminalOptions"));
+    entryMap.remove(QStringLiteral("TerminalOptions"));
     m_strPath = config->readPath();
-    entryMap.remove(QLatin1String("Path"));
+    entryMap.remove(QStringLiteral("Path"));
     m_strComment = config->readComment();
-    entryMap.remove(QLatin1String("Comment"));
+    entryMap.remove(QStringLiteral("Comment"));
     m_strGenName = config->readGenericName();
-    entryMap.remove(QLatin1String("GenericName"));
+    entryMap.remove(QStringLiteral("GenericName"));
     QString _untranslatedGenericName = desktopGroup.readEntryUntranslated("GenericName");
     if (!_untranslatedGenericName.isEmpty()) {
-        entryMap.insert(QLatin1String("UntranslatedGenericName"), _untranslatedGenericName);
+        entryMap.insert(QStringLiteral("UntranslatedGenericName"), _untranslatedGenericName);
     }
 
+    m_lstFormFactors = desktopGroup.readEntry("X-KDE-FormFactors", QStringList());
+    entryMap.remove(QStringLiteral("X-KDE-FormFactors"));
+
     m_lstKeywords = desktopGroup.readXdgListEntry("Keywords", QStringList());
-    entryMap.remove(QLatin1String("Keywords"));
+    entryMap.remove(QStringLiteral("Keywords"));
     m_lstKeywords += desktopGroup.readEntry("X-KDE-Keywords", QStringList());
-    entryMap.remove(QLatin1String("X-KDE-Keywords"));
+    entryMap.remove(QStringLiteral("X-KDE-Keywords"));
     categories = desktopGroup.readXdgListEntry("Categories");
-    entryMap.remove(QLatin1String("Categories"));
+    entryMap.remove(QStringLiteral("Categories"));
     // TODO KDE5: only care for X-KDE-Library in Type=Service desktop files
     // This will prevent people defining a part and an app in the same desktop file
     // which makes user-preference handling difficult.
     m_strLibrary = desktopGroup.readEntry("X-KDE-Library");
-    entryMap.remove(QLatin1String("X-KDE-Library"));
+    entryMap.remove(QStringLiteral("X-KDE-Library"));
     if (!m_strLibrary.isEmpty() && m_strType == QLatin1String("Application")) {
-        qWarning() << "The desktop entry file" << entryPath << "has Type=" << m_strType
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType
                    << "but also has a X-KDE-Library key. This works for now,"
                    " but makes user-preference handling difficult, so support for this might"
                    " be removed at some point. Consider splitting it into two desktop files.";
     }
 
     QStringList lstServiceTypes = desktopGroup.readEntry("ServiceTypes", QStringList());
-    entryMap.remove(QLatin1String("ServiceTypes"));
+    entryMap.remove(QStringLiteral("ServiceTypes"));
     lstServiceTypes += desktopGroup.readEntry("X-KDE-ServiceTypes", QStringList());
-    entryMap.remove(QLatin1String("X-KDE-ServiceTypes"));
+    entryMap.remove(QStringLiteral("X-KDE-ServiceTypes"));
     lstServiceTypes += desktopGroup.readXdgListEntry("MimeType");
-    entryMap.remove(QLatin1String("MimeType"));
+    entryMap.remove(QStringLiteral("MimeType"));
 
     if (m_strType == QLatin1String("Application") && !lstServiceTypes.contains(QLatin1String("Application")))
         // Applications implement the service type "Application" ;-)
     {
-        lstServiceTypes += QString::fromLatin1("Application");
+        lstServiceTypes += QStringLiteral("Application");
     }
 
     m_initialPreference = desktopGroup.readEntry("InitialPreference", 1);
-    entryMap.remove(QLatin1String("InitialPreference"));
+    entryMap.remove(QStringLiteral("InitialPreference"));
 
     // Assign the "initial preference" to each mimetype/servicetype
     // (and to set such preferences in memory from kbuildsycoca)
@@ -214,7 +215,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     while (st_it.hasNext()) {
         const QString st = st_it.next();
         if (st.isEmpty()) {
-            qWarning() << "The desktop entry file" << entryPath << "has an empty mimetype!";
+            qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has an empty mimetype!";
             continue;
         }
         int initialPreference = m_initialPreference;
@@ -235,7 +236,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     }
 
     QString dbusStartupType = desktopGroup.readEntry("X-DBUS-StartupType").toLower();
-    entryMap.remove(QLatin1String("X-DBUS-StartupType"));
+    entryMap.remove(QStringLiteral("X-DBUS-StartupType"));
     if (dbusStartupType == QLatin1String("unique")) {
         m_DBUSStartusType = KService::DBusUnique;
     } else if (dbusStartupType == QLatin1String("multi")) {
@@ -247,14 +248,14 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     m_strDesktopEntryName = _name.toLower();
 
     m_bAllowAsDefault = desktopGroup.readEntry("AllowDefault", true);
-    entryMap.remove(QLatin1String("AllowDefault"));
+    entryMap.remove(QStringLiteral("AllowDefault"));
 
     // allow plugin users to translate categories without needing a separate key
-    QMap<QString, QString>::Iterator entry = entryMap.find(QString::fromLatin1("X-KDE-PluginInfo-Category"));
+    QMap<QString, QString>::Iterator entry = entryMap.find(QStringLiteral("X-KDE-PluginInfo-Category"));
     if (entry != entryMap.end()) {
         const QString &key = entry.key();
         m_mapProps.insert(key, QVariant(desktopGroup.readEntryUntranslated(key)));
-        m_mapProps.insert(key + QLatin1String("-Translated"), QVariant(*entry));
+        m_mapProps.insert(key + QStringLiteral("-Translated"), QVariant(*entry));
         entryMap.erase(entry);
     }
 
@@ -267,7 +268,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         const QString key = it.key();
         // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
         if (!key.contains(QLatin1Char('['))) {
-            //qDebug() << "  Key =" << key << " Data =" << *it;
+            //qCDebug(SERVICES) << "  Key =" << key << " Data =" << *it;
             m_mapProps.insert(key, QVariant(*it));
         }
     }
@@ -292,7 +293,7 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
         if (config->hasActionGroup(group)) {
             const KConfigGroup cg = config->actionGroup(group);
             if (!cg.hasKey("Name") || !cg.hasKey("Exec")) {
-                qWarning() << "The action" << group << "in the desktop file" << q->entryPath() << "has no Name or no Exec key";
+                qCWarning(SERVICES) << "The action" << group << "in the desktop file" << q->entryPath() << "has no Name or no Exec key";
             } else {
                 m_actions.append(KServiceAction(group,
                                                 cg.readEntry("Name"),
@@ -301,7 +302,7 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
                                                 cg.readEntry("NoDisplay", false)));
             }
         } else {
-            qWarning() << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
+            qCWarning(SERVICES) << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
         }
     }
 }
@@ -323,11 +324,12 @@ void KServicePrivate::load(QDataStream &s)
       >> m_strDesktopEntryName
       >> initpref
       >> m_lstKeywords >> m_strGenName
-      >> categories >> menuId >> m_actions >> m_serviceTypes;
+      >> categories >> menuId >> m_actions >> m_serviceTypes
+      >> m_lstFormFactors;
 
-    m_bAllowAsDefault = (bool)def;
-    m_bTerminal = (bool)term;
-    m_DBUSStartusType = (KService::DBusStartupType) dst;
+    m_bAllowAsDefault = bool(def);
+    m_bTerminal = bool(term);
+    m_DBUSStartusType = static_cast<KService::DBusStartupType>(dst);
     m_initialPreference = initpref;
 
     m_bValid = true;
@@ -338,7 +340,7 @@ void KServicePrivate::save(QDataStream &s)
     KSycocaEntryPrivate::save(s);
     qint8 def = m_bAllowAsDefault, initpref = m_initialPreference;
     qint8 term = m_bTerminal;
-    qint8 dst = (qint8) m_DBUSStartusType;
+    qint8 dst = qint8(m_DBUSStartusType);
 
     // WARNING: THIS NEEDS TO REMAIN COMPATIBLE WITH PREVIOUS KService 5.x VERSIONS!
     // !! This data structure should remain binary compatible at all times !!
@@ -352,7 +354,8 @@ void KServicePrivate::save(QDataStream &s)
       << m_strDesktopEntryName
       << initpref
       << m_lstKeywords << m_strGenName
-      << categories << menuId << m_actions << m_serviceTypes;
+      << categories << menuId << m_actions << m_serviceTypes
+      << m_lstFormFactors;
 }
 
 ////
@@ -361,7 +364,7 @@ KService::KService(const QString &_name, const QString &_exec, const QString &_i
     : KSycocaEntry(*new KServicePrivate(QString()))
 {
     Q_D(KService);
-    d->m_strType = QString::fromLatin1("Application");
+    d->m_strType = QStringLiteral("Application");
     d->m_strName = _name;
     d->m_strExec = _exec;
     d->m_strIcon = _icon;
@@ -412,7 +415,8 @@ bool KService::hasServiceType(const QString &serviceType) const
     //if ( serviceOffset == 0 )
     //    serviceOffset = serviceByStorageId( storageId() );
     if (serviceOffset) {
-        return KServiceFactory::self()->hasOffer(ptr->offset(), ptr->serviceOffersOffset(), serviceOffset);
+        KSycoca::self()->ensureCacheValid();
+        return KSycocaPrivate::self()->serviceFactory()->hasOffer(ptr->offset(), ptr->serviceOffersOffset(), serviceOffset);
     }
 
     // fall-back code for services that are NOT from ksycoca
@@ -421,7 +425,7 @@ bool KService::hasServiceType(const QString &serviceType) const
     QVector<ServiceTypeAndPreference>::ConstIterator it = d->m_serviceTypes.begin();
     for (; it != d->m_serviceTypes.end(); ++it) {
         const QString &st = (*it).serviceType;
-        //qDebug() << "    has " << (*it);
+        //qCDebug(SERVICES) << "    has " << (*it);
         if (st == ptr->name()) {
             return true;
         }
@@ -444,20 +448,21 @@ bool KService::hasMimeType(const QString &mimeType) const
     }
     int serviceOffset = offset();
     if (serviceOffset) {
-        KMimeTypeFactory *factory = KMimeTypeFactory::self();
+        KSycoca::self()->ensureCacheValid();
+        KMimeTypeFactory *factory = KSycocaPrivate::self()->mimeTypeFactory();
         const int mimeOffset = factory->entryOffset(mime);
         const int serviceOffersOffset = factory->serviceOffersOffset(mime);
         if (serviceOffersOffset == -1) {
             return false;
         }
-        return KServiceFactory::self()->hasOffer(mimeOffset, serviceOffersOffset, serviceOffset);
+        return KSycocaPrivate::self()->serviceFactory()->hasOffer(mimeOffset, serviceOffersOffset, serviceOffset);
     }
 
     // fall-back code for services that are NOT from ksycoca
     QVector<ServiceTypeAndPreference>::ConstIterator it = d->m_serviceTypes.begin();
     for (; it != d->m_serviceTypes.end(); ++it) {
         const QString &st = (*it).serviceType;
-        //qDebug() << "    has " << (*it);
+        //qCDebug(SERVICES) << "    has " << (*it);
         if (st == mime) {
             return true;
         }
@@ -526,6 +531,8 @@ QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
         return QVariant(categories);
     } else if (_name == QLatin1String("Keywords")) {
         return QVariant(m_lstKeywords);
+    } else if (_name == QLatin1String("FormFactors")) {
+        return QVariant(m_lstFormFactors);
     }
 
     // Ok we need to convert the property from a QString to its real type.
@@ -534,23 +541,23 @@ QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
         // No luck, let's ask KServiceTypeFactory what the type of this property
         // is supposed to be.
         // ######### this looks in all servicetypes, not just the ones this service supports!
-        t = KServiceTypeFactory::self()->findPropertyTypeByName(_name);
+        KSycoca::self()->ensureCacheValid();
+        t = KSycocaPrivate::self()->serviceTypeFactory()->findPropertyTypeByName(_name);
         if (t == QVariant::Invalid) {
-            qDebug() << "Request for unknown property '" << _name << "'\n";
+            qCDebug(SERVICES) << "Request for unknown property" << _name;
             return QVariant(); // Unknown property: Invalid variant.
         }
     }
 
     QMap<QString, QVariant>::ConstIterator it = m_mapProps.find(_name);
     if ((it == m_mapProps.end()) || (!it->isValid())) {
-        //qDebug() << "Property not found " << _name;
+        //qCDebug(SERVICES) << "Property not found " << _name;
         return QVariant(); // No property set.
     }
 
-    switch (t) {
-    case QVariant::String:
+    if (t == QVariant::String) {
         return *it; // no conversion necessary
-    default:
+    } else {
         // All others
         // For instance properties defined as StringList, like MimeTypes.
         // XXX This API is accessible only through a friend declaration.
@@ -567,82 +574,61 @@ QStringList KServicePrivate::propertyNames() const
         res.append(it.key());
     }
 
-    res.append(QString::fromLatin1("Type"));
-    res.append(QString::fromLatin1("Name"));
-    res.append(QString::fromLatin1("Comment"));
-    res.append(QString::fromLatin1("GenericName"));
-    res.append(QString::fromLatin1("Icon"));
-    res.append(QString::fromLatin1("Exec"));
-    res.append(QString::fromLatin1("Terminal"));
-    res.append(QString::fromLatin1("TerminalOptions"));
-    res.append(QString::fromLatin1("Path"));
-    res.append(QString::fromLatin1("ServiceTypes"));
-    res.append(QString::fromLatin1("AllowAsDefault"));
-    res.append(QString::fromLatin1("InitialPreference"));
-    res.append(QString::fromLatin1("Library"));
-    res.append(QString::fromLatin1("DesktopEntryPath"));
-    res.append(QString::fromLatin1("DesktopEntryName"));
-    res.append(QString::fromLatin1("Keywords"));
-    res.append(QString::fromLatin1("Categories"));
+    res.append(QStringLiteral("Type"));
+    res.append(QStringLiteral("Name"));
+    res.append(QStringLiteral("Comment"));
+    res.append(QStringLiteral("GenericName"));
+    res.append(QStringLiteral("Icon"));
+    res.append(QStringLiteral("Exec"));
+    res.append(QStringLiteral("Terminal"));
+    res.append(QStringLiteral("TerminalOptions"));
+    res.append(QStringLiteral("Path"));
+    res.append(QStringLiteral("ServiceTypes"));
+    res.append(QStringLiteral("AllowAsDefault"));
+    res.append(QStringLiteral("InitialPreference"));
+    res.append(QStringLiteral("Library"));
+    res.append(QStringLiteral("DesktopEntryPath"));
+    res.append(QStringLiteral("DesktopEntryName"));
+    res.append(QStringLiteral("Keywords"));
+    res.append(QStringLiteral("FormFactors"));
+    res.append(QStringLiteral("Categories"));
 
     return res;
 }
 
 KService::List KService::allServices()
 {
-    return KServiceFactory::self()->allServices();
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceFactory()->allServices();
 }
 
 KService::Ptr KService::serviceByDesktopPath(const QString &_name)
 {
-    return KServiceFactory::self()->findServiceByDesktopPath(_name);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceFactory()->findServiceByDesktopPath(_name);
 }
 
 KService::Ptr KService::serviceByDesktopName(const QString &_name)
 {
-    return KServiceFactory::self()->findServiceByDesktopName(_name);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceFactory()->findServiceByDesktopName(_name);
 }
 
 KService::Ptr KService::serviceByMenuId(const QString &_name)
 {
-    return KServiceFactory::self()->findServiceByMenuId(_name);
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceFactory()->findServiceByMenuId(_name);
 }
 
 KService::Ptr KService::serviceByStorageId(const QString &_storageId)
 {
-    KService::Ptr service = KService::serviceByMenuId(_storageId);
-    if (service) {
-        return service;
-    }
-
-    service = KService::serviceByDesktopPath(_storageId);
-    if (service) {
-        return service;
-    }
-
-    if (!QDir::isRelativePath(_storageId) && QFile::exists(_storageId)) {
-        return KService::Ptr(new KService(_storageId));
-    }
-
-    QString tmp = _storageId;
-    tmp = tmp.mid(tmp.lastIndexOf(QLatin1Char('/')) + 1); // Strip dir
-
-    if (tmp.endsWith(QLatin1String(".desktop"))) {
-        tmp.truncate(tmp.length() - 8);
-    }
-
-    if (tmp.endsWith(QLatin1String(".kdelnk"))) {
-        tmp.truncate(tmp.length() - 7);
-    }
-
-    service = KService::serviceByDesktopName(tmp);
-
-    return service;
+    KSycoca::self()->ensureCacheValid();
+    return KSycocaPrivate::self()->serviceFactory()->findServiceByStorageId(_storageId);
 }
 
 bool KService::substituteUid() const
 {
-    QVariant v = property(QLatin1String("X-KDE-SubstituteUID"), QVariant::Bool);
+    QVariant v = property(QStringLiteral("X-KDE-SubstituteUID"), QVariant::Bool);
     return v.isValid() && v.toBool();
 }
 
@@ -650,13 +636,13 @@ QString KService::username() const
 {
     // See also KDesktopFile::tryExec()
     QString user;
-    QVariant v = property(QLatin1String("X-KDE-Username"), QVariant::String);
+    QVariant v = property(QStringLiteral("X-KDE-Username"), QVariant::String);
     user = v.isValid() ? v.toString() : QString();
     if (user.isEmpty()) {
         user = QString::fromLocal8Bit(qgetenv("ADMIN_ACCOUNT"));
     }
     if (user.isEmpty()) {
-        user = QString::fromLatin1("root");
+        user = QStringLiteral("root");
     }
     return user;
 }
@@ -669,12 +655,12 @@ bool KService::showInCurrentDesktop() const
     if (currentDesktops.isEmpty()) {
         // This could be an old display manager, or e.g. a failsafe session with no desktop name
         // In doubt, let's say we show KDE stuff.
-        currentDesktops << "KDE";
+        currentDesktops << QStringLiteral("KDE");
     }
 
     // This algorithm is described in the desktop entry spec
 
-    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QString::fromLatin1("OnlyShowIn"));
+    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("OnlyShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
         foreach (const QString &desktop, currentDesktops) {
@@ -685,7 +671,7 @@ bool KService::showInCurrentDesktop() const
         return false;
     }
 
-    it = d->m_mapProps.find(QString::fromLatin1("NotShowIn"));
+    it = d->m_mapProps.find(QStringLiteral("NotShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
         foreach (const QString &desktop, currentDesktops) {
@@ -725,7 +711,7 @@ bool KService::showOnCurrentPlatform() const
 
 bool KService::noDisplay() const
 {
-    if (qvariant_cast<bool>(property(QString::fromLatin1("NoDisplay"), QVariant::Bool))) {
+    if (qvariant_cast<bool>(property(QStringLiteral("NoDisplay"), QVariant::Bool))) {
         return true;
     }
 
@@ -746,14 +732,14 @@ bool KService::noDisplay() const
 
 QString KService::untranslatedGenericName() const
 {
-    QVariant v = property(QString::fromLatin1("UntranslatedGenericName"), QVariant::String);
+    QVariant v = property(QStringLiteral("UntranslatedGenericName"), QVariant::String);
     return v.isValid() ? v.toString() : QString();
 }
 
 QString KService::parentApp() const
 {
     Q_D(const KService);
-    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QLatin1String("X-KDE-ParentApp"));
+    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("X-KDE-ParentApp"));
     if ((it == d->m_mapProps.end()) || (!it->isValid())) {
         return QString();
     }
@@ -764,7 +750,7 @@ QString KService::parentApp() const
 QString KService::pluginKeyword() const
 {
     Q_D(const KService);
-    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QString::fromLatin1("X-KDE-PluginKeyword"));
+    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("X-KDE-PluginKeyword"));
     if ((it == d->m_mapProps.end()) || (!it->isValid())) {
         return QString();
     }
@@ -775,9 +761,9 @@ QString KService::pluginKeyword() const
 QString KService::docPath() const
 {
     Q_D(const KService);
-    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QLatin1String("X-DocPath"));
+    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("X-DocPath"));
     if ((it == d->m_mapProps.end()) || (!it->isValid())) {
-        it = d->m_mapProps.find(QString::fromLatin1("DocPath"));
+        it = d->m_mapProps.find(QStringLiteral("DocPath"));
         if ((it == d->m_mapProps.end()) || (!it->isValid())) {
             return QString();
         }
@@ -839,9 +825,9 @@ QString KService::newServicePath(bool showInMenu, const QString &suggestedName,
     QString result;
     for (int i = 1; true; i++) {
         if (i == 1) {
-            result = base + QString::fromLatin1(".desktop");
+            result = base + QStringLiteral(".desktop");
         } else {
-            result = base + QString::fromLatin1("-%1.desktop").arg(i);
+            result = base + QStringLiteral("-%1.desktop").arg(i);
         }
 
         if (reservedMenuIds && reservedMenuIds->contains(result)) {
@@ -877,7 +863,7 @@ QString KService::exec() const
 {
     Q_D(const KService);
     if (d->m_strType == QLatin1String("Application") && d->m_strExec.isEmpty()) {
-        qWarning() << "The desktop entry file " << entryPath()
+        qCWarning(SERVICES) << "The desktop entry file " << entryPath()
                    << " has Type=" << d->m_strType << " but has no Exec field.";
     }
     return d->m_strExec;
@@ -907,6 +893,12 @@ bool KService::terminal() const
     return d->m_bTerminal;
 }
 
+bool KService::runOnDiscreteGpu() const
+{
+    QVariant v = property(QStringLiteral("X-KDE-RunOnDiscreteGpu"), QVariant::Bool);
+    return v.isValid() && v.toBool();
+}
+
 QString KService::desktopEntryName() const
 {
     Q_D(const KService);
